<?php
/*
Plugin Name: Woo Top Product Word Tags
Description: Generates unique top words per category with additional filtering for colors, sizes, and brands. Admin page for manual updating, exclusion of certain words, and an overall top word list.
Version: 1.9
Author: EasyToolsWP
*/

// Register a cron job for daily updating
register_activation_hook(__FILE__, 'top_word_tags_activation');
function top_word_tags_activation() {
    if (!wp_next_scheduled('update_top_word_tags_daily')) {
        wp_schedule_event(time(), 'daily', 'update_top_word_tags_daily');
    }
}

// Remove the cron job on deactivation
register_deactivation_hook(__FILE__, 'top_word_tags_deactivation');
function top_word_tags_deactivation() {
    wp_clear_scheduled_hook('update_top_word_tags_daily');
}

// Add admin menu page for managing tags
add_action('admin_menu', 'top_word_tags_menu');
function top_word_tags_menu() {
    add_menu_page(
        'Top Word Tags for Categories',
        'Top Word Tags',
        'manage_options',
        'top-word-tags',
        'top_word_tags_admin_page',
        'dashicons-tag',
        20
    );
}

// Admin page to view and update top words
function top_word_tags_admin_page() {
    // Fetch all product categories
    $categories = get_terms(array(
        'taxonomy' => 'product_cat',
        'hide_empty' => false,
    ));

// Hvis form er submitted, opdater word list
if ($_SERVER['REQUEST_METHOD'] == 'POST' && isset($_POST['update_top_words'])) {
    $excluded_words = sanitize_text_field($_POST['excluded_words']);
    $color_words = sanitize_text_field($_POST['color_words']);
    $size_words = sanitize_text_field($_POST['size_words']);
    $brand_words = sanitize_text_field($_POST['brand_words']);
    
    update_option('excluded_words', $excluded_words);
    update_option('color_words', $color_words);
    update_option('size_words', $size_words);
    update_option('brand_words', $brand_words);

    $number_of_words = intval($_POST['number_of_words']);
    update_option('number_of_words', $number_of_words);
    
    foreach ($categories as $category) {
        update_top_words_for_category($category);
    }

    // Tjek om funktionen eksisterer før kaldet til `update_overall_top_words()`
    if (function_exists('update_overall_top_words')) {
        update_overall_top_words();
    }

    echo '<div class="updated"><p>Top words have been updated for all categories and overall list.</p></div>';
}

    // Fetch excluded words, color words, size words, brand words, and number of top words
    $excluded_words = get_option('excluded_words', '');
    $color_words = get_option('color_words', '');
    $size_words = get_option('size_words', '');
    $brand_words = get_option('brand_words', '');
    $number_of_words = get_option('number_of_words', 10); // Default is 10

    // Admin form
    echo '<div class="wrap">';
    echo '<h1>Top Word Tags for Categories</h1>';
    echo '<form method="post">';
    echo '<textarea name="excluded_words" rows="5" cols="50" placeholder="Enter words to exclude, separated by commas...">' . esc_textarea($excluded_words) . '</textarea>';
    echo '<br><br>';
    echo '<textarea name="color_words" rows="5" cols="50" placeholder="Enter color words, separated by commas...">' . esc_textarea($color_words) . '</textarea>';
    echo '<br><br>';
    echo '<textarea name="size_words" rows="5" cols="50" placeholder="Enter size words, separated by commas...">' . esc_textarea($size_words) . '</textarea>';
    echo '<br><br>';
    echo '<textarea name="brand_words" rows="5" cols="50" placeholder="Enter brand words, separated by commas...">' . esc_textarea($brand_words) . '</textarea>';
    echo '<br><br>';
    echo '<label for="number_of_words">Number of words to retrieve per category:</label>';
    echo '<input type="number" name="number_of_words" value="' . esc_attr($number_of_words) . '" min="1" max="50">';
    echo '<br><br><input type="submit" name="update_top_words" class="button button-primary" value="Update Top Words for Categories and Overall List">';
    echo '</form>';

    // Display existing words for each category
    echo '<h2>Existing Top Words for Categories</h2>';
    foreach ($categories as $category) {
        $top_words = get_term_meta($category->term_id, 'top_word_tags', true);
        $colors = get_term_meta($category->term_id, 'color_tags', true);
        $sizes = get_term_meta($category->term_id, 'size_tags', true);
        $brands = get_term_meta($category->term_id, 'brand_tags', true);

        if (is_array($top_words) && !empty($top_words)) {
            echo '<h3>' . esc_html($category->name) . '</h3>';
            echo '<p><strong>Top Words:</strong> ' . implode(', ', array_keys($top_words)) . '</p>';
            echo '<p><strong>Colors:</strong> ' . (!empty($colors) ? implode(', ', array_keys($colors)) : 'No color tags found.') . '</p>';
            echo '<p><strong>Sizes:</strong> ' . (!empty($sizes) ? implode(', ', array_keys($sizes)) : 'No size tags found.') . '</p>';
            echo '<p><strong>Brands:</strong> ' . (!empty($brands) ? implode(', ', array_keys($brands)) : 'No brand tags found.') . '</p>';
        } else {
            echo '<h3>' . esc_html($category->name) . '</h3>';
            echo '<p>No words found for this category.</p>';
        }
    }

    // Display the overall list of top words
    echo '<h2>Overall Top Words for All Products</h2>';
    $overall_top_words = get_option('overall_top_word_tags', array());
    if (is_array($overall_top_words) && !empty($overall_top_words)) {
        echo '<p>' . implode(', ', array_keys($overall_top_words)) . '</p>';
    } else {
        echo '<p>No overall top words found.</p>';
    }
    echo '</div>';
}

// Update top words for a specific category, separating general, color, size, and brand words
function update_top_words_for_category($category) {
    $args = array(
        'post_type' => 'product',
        'posts_per_page' => -1,
        'tax_query' => array(
            array(
                'taxonomy' => 'product_cat',
                'field' => 'term_id',
                'terms' => $category->term_id,
            ),
        ),
    );
    $products = get_posts($args);

    $word_count = array();
    $color_count = array();
    $size_count = array();
    $brand_count = array();

    $excluded_words = explode(',', get_option('excluded_words', ''));
    $color_words = explode(',', get_option('color_words', ''));
    $size_words = explode(',', get_option('size_words', ''));
    $brand_words = explode(',', get_option('brand_words', ''));

    $excluded_words = array_map('trim', array_map('mb_strtolower', $excluded_words));
    $color_words = array_map('trim', array_map('mb_strtolower', $color_words));
    $size_words = array_map('trim', array_map('mb_strtolower', $size_words));
    $brand_words = array_map('trim', array_map('mb_strtolower', $brand_words));

    foreach ($products as $product) {
        $product_name = $product->post_title;
        $words = explode(' ', $product_name);
        foreach ($words as $word) {
            $word = mb_strtolower(preg_replace('/[^a-zA-ZæøåÆØÅ0-9]/u', '', $word));
            if (strlen($word) > 2 && !in_array($word, $excluded_words)) {
                if (in_array($word, $color_words)) {
                    if (array_key_exists($word, $color_count)) {
                        $color_count[$word]++;
                    } else {
                        $color_count[$word] = 1;
                    }
                } elseif (in_array($word, $size_words)) {
                    if (array_key_exists($word, $size_count)) {
                        $size_count[$word]++;
                    } else {
                        $size_count[$word] = 1;
                    }
                } elseif (in_array($word, $brand_words)) {
                    if (array_key_exists($word, $brand_count)) {
                        $brand_count[$word]++;
                    } else {
                        $brand_count[$word] = 1;
                    }
                } else {
                    if (array_key_exists($word, $word_count)) {
                        $word_count[$word]++;
                    } else {
                        $word_count[$word] = 1;
                    }
                }
            }
        }
    }

    arsort($word_count);
    arsort($color_count);
    arsort($size_count);
    arsort($brand_count);

    $number_of_words = get_option('number_of_words', 10);
    $top_words = array_slice($word_count, 0, $number_of_words, true);
    $top_colors = array_slice($color_count, 0, $number_of_words, true);
    $top_sizes = array_slice($size_count, 0, $number_of_words, true);
    $top_brands = array_slice($brand_count, 0, $number_of_words, true);

    // Save the lists as category metadata
    update_term_meta($category->term_id, 'top_word_tags', $top_words);
    update_term_meta($category->term_id, 'color_tags', $top_colors);
    update_term_meta($category->term_id, 'size_tags', $top_sizes);
    update_term_meta($category->term_id, 'brand_tags', $top_brands);
}

// Shortcode to display top words, colors, sizes, or brands
add_shortcode('topproductword', 'display_top_product_words_shortcode');
// Generate a responsive tag display with mobile-specific tag limits
// Generate a responsive tag display with mobile-specific tag limits
function display_top_product_words_shortcode($atts) {
    // Parse shortcode attributes
    $atts = shortcode_atts(
        array(
            'items' => 10,         // Default number of words to display on desktop
            'mobile_items' => 5,   // Default number of words to display on mobile
            'category' => '',      // Default value for category
            'type' => 'topwords',  // Default to topwords (can be 'brands', 'colors', 'sizes')
        ),
        $atts,
        'topproductword'
    );

    $items = intval($atts['items']);
    $mobile_items = intval($atts['mobile_items']);
    $category = sanitize_text_field($atts['category']);
    $type = sanitize_text_field($atts['type']);
    $top_words = [];

    // Fetch metadata for the specified category
    if (!empty($category)) {
        $category_obj = get_term_by('slug', $category, 'product_cat');
        if (!$category_obj) {
            $category_obj = get_term_by('name', $category, 'product_cat');
        }

        if ($category_obj) {
            switch ($type) {
                case 'brands':
                    $top_words = get_term_meta($category_obj->term_id, 'brand_tags', true);
                    break;
                case 'colors':
                    $top_words = get_term_meta($category_obj->term_id, 'color_tags', true);
                    break;
                case 'sizes':
                    $top_words = get_term_meta($category_obj->term_id, 'size_tags', true);
                    break;
                default:
                    $top_words = get_term_meta($category_obj->term_id, 'top_word_tags', true);
                    break;
            }
        } else {
            return '<p>Category not found.</p>';
        }
    } else {
        switch ($type) {
            case 'brands':
                $top_words = get_option('overall_brand_tags', array());
                break;
            case 'colors':
                $top_words = get_option('overall_color_tags', array());
                break;
            case 'sizes':
                $top_words = get_option('overall_size_tags', array());
                break;
            default:
                $top_words = get_option('overall_top_word_tags', array());
                break;
        }
    }

    if (is_array($top_words) && !empty($top_words)) {
        // Inline CSS for styling the tags
        $output = '<style>
            .top-word-tags {
                display: flex;
                flex-wrap: wrap;
                gap: 5px;
            }
            .top-word-tags .tag-bubble {
                background-color: #f1f1f1;
                color: #333;
                padding: 5px 10px;
                border-radius: 20px;
                font-size: 14px;
                text-decoration: none;
                display: inline-block;
            }
            /* Hide additional tags on mobile */
            @media (max-width: 768px) {
                .top-word-tags .tag-bubble:nth-child(n+' . ($mobile_items + 1) . ') {
                    display: none;
                }
				.top-word-section, .top-attributes-container one-section {
				display:none;
				}
            }
        </style>';

        // HTML container for tags
        $output .= '<div class="top-word-tags">';

        // Limit tags to display up to $items for desktop and up to $mobile_items for mobile
        $counter = 0;
        foreach ($top_words as $word => $count) {
            $search_url = add_query_arg(array(
                's' => $word,
                'post_type' => 'product',
            ), home_url('/'));

            $output .= '<a href="' . esc_url($search_url) . '" class="tag-bubble">' . esc_html($word) . '</a>';
            $counter++;
            if ($counter >= $items) {
                break; // Limit tags to the desktop count specified
            }
        }
        $output .= '</div>';

        return $output;
    } else {
        return '<p>No top words found.</p>';
    }
}
add_shortcode('top_product_words', 'display_top_product_words_shortcode');




// Display top words, colors, sizes, and brands on product category pages
add_action('woocommerce_before_shop_loop', 'display_top_10_word_tags');
function display_top_10_word_tags() {
    if (is_product_category()) {
        $current_category = get_queried_object();
        $top_words = get_term_meta($current_category->term_id, 'top_word_tags', true);
        $colors = get_term_meta($current_category->term_id, 'color_tags', true);
        $sizes = get_term_meta($current_category->term_id, 'size_tags', true);
        $brands = get_term_meta($current_category->term_id, 'brand_tags', true);

        // Display Top Words separately
        if (is_array($top_words) && !empty($top_words)) {
            echo '<div class="top-word-section">';
            foreach ($top_words as $word => $count) {
                $search_url = add_query_arg(array(
                    's' => $word,
                    'post_type' => 'product',
                    'product_cat' => $current_category->slug,
                ), home_url('/'));

                echo '<a href="' . esc_url($search_url) . '" class="tag-bubble">' . esc_html($word) . '</a>';
            }
            echo '</div>';
        }

        // Collect sections for Colors, Sizes, and Brands
        $sections = []; // Hold all non-empty sections

        if (is_array($colors) && !empty($colors)) {
            $content = '<h3>Colors</h3>';
            foreach ($colors as $word => $count) {
                $search_url = add_query_arg(array(
                    's' => $word,
                    'post_type' => 'product',
                    'product_cat' => $current_category->slug,
                ), home_url('/'));

                $content .= '<a href="' . esc_url($search_url) . '" class="tag-bubble">' . esc_html($word) . '</a>';
            }
            $sections[] = '<div class="top-attribute-section">' . $content . '</div>';
        }

        if (is_array($sizes) && !empty($sizes)) {
            $content = '<h3>Sizes</h3>';
            foreach ($sizes as $word => $count) {
                $search_url = add_query_arg(array(
                    's' => $word,
                    'post_type' => 'product',
                    'product_cat' => $current_category->slug,
                ), home_url('/'));

                $content .= '<a href="' . esc_url($search_url) . '" class="tag-bubble">' . esc_html($word) . '</a>';
            }
            $sections[] = '<div class="top-attribute-section">' . $content . '</div>';
        }

        if (is_array($brands) && !empty($brands)) {
            $content = '<h3>Brands</h3>';
            foreach ($brands as $word => $count) {
                $search_url = add_query_arg(array(
                    's' => $word,
                    'post_type' => 'product',
                    'product_cat' => $current_category->slug,
                ), home_url('/'));

                $content .= '<a href="' . esc_url($search_url) . '" class="tag-bubble">' . esc_html($word) . '</a>';
            }
            $sections[] = '<div class="top-attribute-section">' . $content . '</div>';
        }

        // Determine the class for the container based on the number of sections
        $section_count = count($sections);
        $container_class = 'top-attributes-container';
        if ($section_count === 3) {
            $container_class .= ' three-sections';
        } elseif ($section_count === 2) {
            $container_class .= ' two-sections';
        } elseif ($section_count === 1) {
            $container_class .= ' one-section';
        }

        // Display Colors, Sizes, and Brands in a row
        if (!empty($sections)) {
            echo '<div class="' . esc_attr($container_class) . '">';
            echo implode("\n", $sections);
            echo '</div>';
        }
    }
}

// Add CSS to style the tags and layout for colors, sizes, and brands in a row
add_action('wp_head', 'add_custom_tag_styles');
function add_custom_tag_styles() {
    echo '
    <style>
.top-word-section {
    margin-bottom: 20px;
}

.top-attributes-container {
    display: flex;
    gap: 20px;
    margin: 20px 0;
    flex-wrap: wrap;
}

/* Different widths based on the number of sections */
.top-attributes-container.three-sections .top-attribute-section {
    flex: 1 1 calc(33.333% - 20px); /* Each takes 33% with space between */
}

.top-attributes-container.two-sections .top-attribute-section {
    flex: 1 1 calc(50% - 20px); /* Each takes 50% with space between */
}

.top-attributes-container.one-section .top-attribute-section {
    flex: 1 1 100%; /* Takes full width */
}

/* Default styling to apply in case classes are not applied */
.top-attribute-section {
    background-color: #fff;
    padding: 15px;
    border: 1px solid #e0e0e0;
    border-radius: 5px;
    transition: all 0.3s ease;
}



/* Responsive styling for mobile devices */
@media (max-width: 768px) {
    .top-attributes-container {
        flex-direction: column;
    }

    .top-attribute-section {
        width: 100%; /* Ensure full width on mobile */
        margin-bottom: 10px; /* Add margin-bottom for spacing between sections on mobile */
    }
}

.top-word-section h3,
.top-attribute-section h3 {
    margin-top: 0;
    font-size: 18px;
}

.tag-bubble {
    display: inline-block;
    background-color: #f0f0f0; /* Light gray color */
    border-radius: 4px;
    padding: 8px 12px; /* Internal spacing */
    margin: 5px; /* Space between boxes */
    font-size: 14px; /* Standard text size */
    box-shadow: 0 2px 4px rgba(0, 0, 0, 0.1); /* Light shadow for depth */
    text-decoration: none;
    color: #333; /* Dark gray text color */
    cursor: pointer;
    transition: background-color 0.2s, transform 0.2s;
}

.tag-bubble:hover {
    background-color: #e0e0e0; /* Slightly darker gray on hover */
    transform: scale(1.05); /* Slightly larger on hover */
}

    </style>
    ';
}
?>
